@@ -1,11 +1,11 @@
 #: PerlMaple.pm
 #: implementation for the PerlMaple class
-#: v0.02
-#: Copyright (c) 2005 Agent Zhang
-#: 2005-11-14 2005-12-19
+#: Copyright (c) 2005-2006 Agent Zhang
+#: 2005-11-14 2006-02-06
 
 package PerlMaple;
 
+use 5.006;
 use strict;
 use warnings;
 
@@ -13,7 +13,7 @@ use PerlMaple::Expression;
 use Carp qw(carp croak);
 use vars qw( $AUTOLOAD );
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 my $Started = 0;
 
 require XSLoader;
@@ -38,7 +38,9 @@ sub eval_cmd {
     #if ($exp !~ /[;:]$/) {
     #    $exp .= ';';
     #}
-    #warn $exp;
+    if ($self->{LogCmd}) {
+        warn "$exp\n";
+    }
     maple_eval($exp);
     if (maple_success()) {
         my $res = maple_result();
@@ -46,10 +48,12 @@ sub eval_cmd {
             $self->{ReturnAST} ? $self->to_ast($res) : $res;
     }
     if ($self->{PrintError}) {
-        carp "PerlMaple error: ", $self->error;
+        carp "PerlMaple: ", $self->error,
+            " when evaluating \"$exp\"";
     }
     if ($self->{RaiseError}) {
-        croak "PerlMaple error: ", $self->error;
+        croak "PerlMaple: ", $self->error,
+            " when evaluating \"$exp\"";
     }
     return undef;
 }
@@ -97,7 +101,7 @@ sub AUTOLOAD {
     my $self = shift;
     my $method = $AUTOLOAD;
     #warn "$method";
-    
+
     $method =~ s/.*:://;
     return if $method eq 'DESTROY';
 
@@ -149,7 +153,7 @@ PerlMaple - Perl binding for Waterloo's Maple software
   # Advanced usage (manipulating Maple ASTs directly):
   $ast = $maple->to_ast('[1,2,3]');
   foreach ($ast->ops) {
-      print $_->expr;  # got 1, 2, and 3 respectively
+      print;  # got 1, 2, and 3 respectively
   }
 
   # Get eval_cmd and other AUTOLOADed Maple function
@@ -158,14 +162,21 @@ PerlMaple - Perl binding for Waterloo's Maple software
   $ast = $maple->solve('x^2+x=0', 'x');
   if ($ast and $ast->type('exprseq')) {
     foreach ($ast->ops) {
-        push @roots, $_->expr;
+        push @roots, $_;
     }
   }
   print "@roots";  # got: 0 -1
 
+  $maple->ReturnAST(1);
+  $res = $maple->solve('{x+y=2,x-y=3}', '{x,y}');
+  if ($res->type('set')) {
+      $x = first { $_->lhs eq 'x' } $res->ops;
+      print $x->rhs;  # 5/2
+  }
+
 =head1 VERSION
 
-This document describes PerlMaple 0.02 released on December 19, 2005.
+This document describes PerlMaple 0.03 released on February 5, 2006.
 
 =head1 DESCRIPTION
 
@@ -174,16 +185,17 @@ C interface. To use this library, you have to purchase the Maple software:
 
 L<http://www.maplesoft.com>.
 
-Unfortunately, he Maple software is *not* free, sorry, unlike this CPAN 
+Unfortunately, he Maple software is *not* free, sorry, unlike this CPAN
 distribution.
 
 =head1 INSTALLATION
 
 Currently this software is only tested on Win32. To build this module
-properly, you must first have Maple installed on your system and append
+properly, you must first have Maple 9 or better installed on your
+system and append
 the paths of B<maplec.h> and B<maplec.lib> in your Maple installation to the
 environments LIB and INC respectively. Because this module use Maple's
-C interface via L<Inline::C>.
+C interface (OpenMaple) via XS.
 
 Both F<maplec.h> and F<maplec.lib> (or maplec.a on *NIX systems?) are
 provided by your Maple installation itself. A typical path of maplec.h
@@ -195,6 +207,9 @@ machine but do depend on your Maple's version and location.
 
 It may be similar on UNIX, but I haven't tried that.
 
+It's known that this library's XS part won't work with Maple 7 on Win32.
+Sigh.
+
 =head2 EXPORT
 
 None by default.
@@ -202,11 +217,11 @@ None by default.
 =head1 METHODS
 
 PerlMaple uses AUTOLOAD mechanism so that any Maple functions are also valid
-methods of your PerlMaple object, even those Maple procedures defined by 
+methods of your PerlMaple object, even those Maple procedures defined by
 yourself.
 
 When the ReturnAST attribute is on, all AUTOLOADed methods, along with the
--E<gt>eval_cmd method will return a 
+-E<gt>eval_cmd method will return a
 PerlMaple::Expression object constructed from the resulting expression.
 Because there's a cost involved in constructing an AST from the given Maple
 expression, so the ReturnAST attribute is off by default.
@@ -240,9 +255,9 @@ If an error occurs, it will return undef, and set the internal error buffer
 which you can read by the -E<gt>error() method.
 
 Frankly speaking, most of the time you can use -E<gt>eval method instead of
-invoking this method directly. However, this method is a bit faster, 
+invoking this method directly. However, this method is a bit faster,
 because any AUTOLOADed method is invoked this one internally. Moreover, there
-exists something that can only be eval'ed properly by -E<gt>eval_cmd. 
+exists something that can only be eval'ed properly by -E<gt>eval_cmd.
 Here is a small example:
 
     $maple->eval_cmd(<<'.');
@@ -259,8 +274,8 @@ If you use -E<gt>eval instead, you will get the following error message:
 That's because "eval" is a normal Maple function and hence you can't use
 assignment statement as the argument.
 
-When the ReturnAST attribute is on, this method will return a 
-PerlMaple::Expression object constructed from the expression returned by 
+When the ReturnAST attribute is on, this method will return a
+PerlMaple::Expression object constructed from the expression returned by
 Maple automatically.
 
 =item -E<gt>to_ast($maple_expr, ?$verified)
@@ -297,11 +312,11 @@ the -E<gt>new method.
 
 =item -E<gt>PrintError($new_value)
 
-The PrintError attribute can be used to force errors to generate warnings 
+The PrintError attribute can be used to force errors to generate warnings
 (using Carp::carp) in addition to returning error codes in the normal way. When
-set ``on'' (say, a true value), any method which results in an error 
-occurring will cause the PerlMaple to effectively do a 
-C<carp("PerlMaple error: ", $self->error)>.
+set ``on'' (say, a true value), any method which results in an error
+occurring will cause the PerlMaple to effectively do a
+C<carp("PerlMaple error: ", $self->error, " when evaluating \"$exp\"";)>.
 
 By default, PerlMaple-E<gt>new PrintError ``on''.
 
@@ -311,11 +326,11 @@ By default, PerlMaple-E<gt>new PrintError ``on''.
 
 The RaiseError attribute can be used to force errors to raise exceptions
 rather than simply return error codes in the normal way. It is ``off''
-(say, a false value in Perl) by default. When set ``on'', any method 
+(say, a false value in Perl) by default. When set ``on'', any method
 which results in an error will cause the PerlMaple to effectively do a
-C<croak("PerlMaple error: ", $self->error)>.
+C<croak("PerlMaple error: ", $self->error, " when evaluating \"$exp\"";)>.
 
-If you turn RaiseError on then you'd normally turn PrintError off. 
+If you turn RaiseError on then you'd normally turn PrintError off.
 If PrintError is also on, then the PrintError is done first (naturally).
 
 =item -E<gt>ReturnAST
@@ -359,38 +374,75 @@ Indicates whether the last Maple evaluation is successful.
 
 =head1 CODE COVERAGE
 
-I use L<Devel::Cover> to test the code coverage of my tests, below is the 
-L<Devel::Cover> report on this module's test suite (version 0.02):
+I use L<Devel::Cover> to test the code coverage of my tests, below is the
+L<Devel::Cover> report on this module's test suite (version 0.03):
 
     ---------------------------- ------ ------ ------ ------ ------ ------ ------
     File                           stmt   bran   cond    sub    pod   time  total
     ---------------------------- ------ ------ ------ ------ ------ ------ ------
-    blib/lib/PerlMaple.pm          94.8   86.4   66.7  100.0  100.0   98.1   93.2
-    ...b/PerlMaple/Expression.pm  100.0   94.4   66.7  100.0  100.0    1.9   95.1
-    Total                          97.1   90.0   66.7  100.0  100.0  100.0   94.1
+    blib/lib/PerlMaple.pm          93.3   83.3   66.7  100.0  100.0   90.0   91.6
+    ...b/PerlMaple/Expression.pm   98.5   90.0   77.8  100.0  100.0   10.0   95.1
+    Total                          96.1   87.0   75.0  100.0  100.0  100.0   93.4
     ---------------------------- ------ ------ ------ ------ ------ ------ ------
 
+=head1 BUGS
+
+=over
+
+At this point, every PerlMaple objects are merely a bunch of options (e.g.
+PrintError, RaiseError, and ReturnAST). They actually share the same Maple
+engine for performance reasons. So the evaluating contexts for each
+PerlMaple objects may corrupt together. So please don't store
+permanent values in Maple variables, avoid reference Maple special variables,
+like %, %%, and don't trust the Maple contexts. Since they may be changed
+accidentally by other PerlMaple objects, even those used by
+PerlMaple::Expression instances, in the duration.
+
+=back
+
+=head1 TODO
+
+=over
+
+=item *
+
+Add facilities to ease importing of Perl complex data structures to Maple
+environment, i.e. a list of lists of numbers.
+
+=item *
+
+Add the C<LogCommand> attribute so that PerlMaple will log down all the Maple 
+commands sent to the Maple engine in disk files.
+
+=item *
+
+Added PerlMaple::Cookbook and PerlMaple::Tutorial to the distribution.
+Programming Maple in Perl is extremely interesting and useful, so I
+believe these docs are really worth my while.
+
+=back
+
 =head1 REPOSITORY
 
 You can always get the latest version from the following SVN
 repository:
 
-    L<https://svn.berlios.de/svnroot/repos/win32maple>
+L<https://svn.berlios.de/svnroot/repos/win32maple>
 
 If you want a committer bit, please let me know.
 
 =head1 SEE ALSO
 
-L<http://www.maplesoft.com>,
-L<PerlMaple::Expression>.
+L<PerlMaple::Expression>,
+L<http://www.maplesoft.com>.
 
 =head1 AUTHOR
 
-Agent Zhang, E<lt>agent2002@126.comE<gt>
+Agent Zhang, E<lt>agentzh@gmail.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005 Agent Zhang
+Copyright (C) 2005-2006 Agent Zhang
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
